import java.net.*;
import java.util.*;
import java.io.*;

/*----------------------------------------------------*/
/* JavaTcps                                           */
/*  This class provides server support for raffle     */
/*  It establishes a server socket and then accepts   */
/*    requests from socket clients to perform various */
/*    raffle functions. The requests are passed to    */
/*    the javaServData object for execution           */
/*----------------------------------------------------*/
/* Modification List                                  */
/*    07-19-1997      Base version                    */
/*    07-20-1997 tw   attempt to add backup thread    */
/*----------------------------------------------------*/

public class JavaTcps {
   public static void main (String args[]) {
      int xPort = 2300;               // (default) port number
      Socket soClient;                //client socket
      ServerSocket soServer;          //server socket
      boolean bContinue=true;         //global continuation switch
      boolean bStream=true;           //local stream continuation switch
      StringBuffer sb = new StringBuffer(2048);
      int ch;
      int ct=0;
      int ctMax=8000;
      String inputLine;
      InetAddress inet;
      String sHdr1 = "http/1.0 200 OK";
      String sHdr2 = "Content-type: text/html";
      Vector gIn = new Vector(10);
      Vector gOut = new Vector(100);
      javaServData jsrv;
      BackupThread bthread;

      if (args.length > 0)
        xPort = new Integer(args[0]).intValue();

      if (args.length > 1) 
        jsrv = new javaServData(args[1]);
      else
        jsrv = new javaServData();
     
      bthread = new BackupThread(jsrv);
      bthread.start();

      try {
         soServer = new ServerSocket(xPort, 8);     //create server socket
         inet = soServer.getInetAddress();
         System.out.println("listening on " + soServer.getLocalPort());
         System.out.println("address " + inet.getHostAddress());

         while (bContinue) {     
           soClient = soServer.accept();              //wait on client connecting
           System.out.println("connection accepted " + soClient.getLocalPort());
           bStream = true;
           PrintStream outs = new PrintStream(
                                   new BufferedOutputStream(soClient.getOutputStream(), 2048),
                                   false);
           DataInputStream ins = new DataInputStream(soClient.getInputStream());
           gIn.removeAllElements();

           while ( bStream ) {
             inputLine = ins.readLine();
             gIn.addElement(inputLine);
             bStream = (inputLine.length() > 1);
           }

//if POST method, read actual input parameters also

          inputLine = ins.readLine();
          System.out.println(inputLine);
          gOut.removeAllElements();
          gOut = jsrv.processRequest(inputLine);

// end of POST input

           outs.println(sHdr1);
           outs.println(sHdr2);
           outs.println();

           Enumeration enum = gOut.elements();
           while (enum.hasMoreElements()) {
             outs.println( "==" + (String) enum.nextElement() );
           }

           outs.flush();          
           outs.close();
           ins.close();

           soClient.close();
           ct++;
           if (ct > ctMax)
             bContinue = false;

         }
         soServer.close();
      } /* endtry */
      catch (Exception e  ) {
         System.out.println(e.toString());
      } /* endcatch */

      System.out.println("test program ending");
  }

}

class BackupThread extends Thread {      
      private long waitMinutes;
      private long waitTime;
      boolean bContinue    = true;
      javaServData servReq;
      Vector vStatus;
      String myId = "Backup thread: ";

      public BackupThread(javaServData server) {
         servReq = server;
      }

      public void setDelayMinutes(long min) {
         waitMinutes = min;
         waitTime    = waitMinutes * 60000;
         bkMessage("delay set to " + waitMinutes + " (minutes)");
      }

      public void run() {
         bkMessage("started...");
         setDelayMinutes(5);

         while (bContinue) {
           delay(waitTime);
           bkMessage("wakeup ");
           vStatus = servReq.processRequest("Action=flush");
           bkMessage((String) vStatus.elementAt(0));
         }
      }

      private void bkMessage(String msg) {
         Date curDate = new Date();
         String fmtTime = curDate.getHours() + ":" + curDate.getMinutes() + ":" 
                          + curDate.getSeconds() + " ";
         System.out.println(fmtTime + myId + msg);
      }

      private synchronized void delay(long delayIntv) {
         try {
              wait (waitTime);
         }
         catch ( Exception e ) {

         } /* endcatch */
      }
}
