import java.util.*;
import java.io.*;
import java.net.*;

/*------------------------------------------------------*/
/* Java Class Application to emulate REXX Server Module */
/*  this version provides functions needed by server    */
/*   interface and is invoked by TCPIP server module    */
/*   (or can also be invoked by standalon version)      */
/*------------------------------------------------------*/

public class javaServData {
   private String memberFileName = "members.txt";
   private String registeredFileName = "reg.txt";
   private Vector allMembers = new Vector(200);
   private Vector regMembers = new Vector(100);
   private String sRequest;
   private boolean bProcess;
   private int    refrMax = 16;
   private int    refrCount = 0;
   private boolean bRegisterChanged=false;
   public  Vector vOut = new Vector(100);

//  constructor for normal startup
   public javaServData() {
        readFile(memberFileName, allMembers);
   }

//  constructor for restart (loads previously registered members)
   public javaServData(String primeFile) {
        readFile(memberFileName, allMembers);
        readFile(primeFile, regMembers);
        bRegisterChanged = true;
   }

//  method to process request
   public Vector processRequest(String request) {
        String reqType="none", reqOperand="", reqFirst="0";
        String token1;

        vOut.removeAllElements();

        StringTokenizer st = new StringTokenizer(request, "&");
        while (st.hasMoreTokens()) {
           token1 = st.nextToken();
           if (token1.startsWith("Action="))
                reqType = getTokenValue(token1, "Action=");
           if (token1.startsWith("members="))
                reqOperand = getTokenValue(token1, "members=");
           if (token1.startsWith("first="))
                reqOperand = getTokenValue(token1, "first=");
        }

        outMessage("reqtype=" + reqType + ",operand=" + reqOperand + ".");

        if (reqType.equals("getall")) {
           returnAll();
           return vOut;
        }

        if (reqType.equals("register")) {
           registerMember(reqOperand);
           return vOut;
        }

        if (reqType.equals("registerguest")) {
           registerGuest(reqOperand);
           return vOut;
        }

        if (reqType.equals("getregistered")) {
           returnRegistered(reqOperand);
           return vOut;
        }

        if (reqType.equals("addmember")) {
           addMember(reqOperand);
           return vOut;
        }

        if (reqType.equals("unregister")) {
           removeRegistered(reqOperand);
           return vOut;
        }

        if (reqType.equals("flush")) {
           if (bRegisterChanged) {
                writeData(registeredFileName, regMembers);
                bRegisterChanged = false;
           }
           else
                vOut.addElement("no changes since last write");
           return vOut;
        }

        if (reqType.equals("quit")) {
           writeData(registeredFileName, regMembers);
           System.exit(0);
        }

        outData("invalid request; not processed");
        return vOut;
   }

//  method to return all members
   public void returnAll() {
        for (int i=0; i < allMembers.size(); i++) {
           outData( (String) allMembers.elementAt(i) );
        }
   }

//  method to register a member
   public void registerMember(String member) {
        String sMsg;

        int x = findElement(allMembers, member);
        if (x > -1) {
           int x2 = findElement(regMembers, member);
           if (x2 == -1) {
              regMembers.addElement(member);
              sMsg = "0:" + member + " registered";
              bRegisterChanged = true;
           }
           else
              sMsg = "10: " + member + " already registered";
        }
        else
           sMsg = "11: " + member + " not a member";

        outData(sMsg);
   }

//  method to register a guest
   public void registerGuest(String guest) {
        String sMsg;
        int x = findElement(regMembers, guest);

        if (x > -1)
                sMsg = "11: " + guest + " already registered";
        else {
                regMembers.addElement(guest);
                sMsg = guest + " added as guest";
                bRegisterChanged = true;
        }
        outData(sMsg);
   }

//  method to return registered 
   public void returnRegistered(String first) {
        int ifirst = 0;

        if (first.length() > 0) 
           ifirst = (new Integer(first)).intValue();

        for (int i=ifirst; i < regMembers.size(); i++) {
           outData( (String) regMembers.elementAt(i) );
        }

   }

//  method to add a new member to the incore list
   public void addMember(String member) {
        int x = findElement(allMembers, member);
        if (x == -1) {
           allMembers.addElement(member);
           outData(member + " added to member list");
        }
        else {
           outData("12: " + member + " already a member");
        }
   }

//  method to remove a registered entry
   public void removeRegistered(String member) {
        int x = findElement(regMembers, member);
        if (x > -1) {
           regMembers.removeElementAt(x);
           outData("0:" + member + " removed from registered list");
           bRegisterChanged = true;
        } 
        else {
           outData("13: " + member + " not registered");
        }
   }

//  meethod to return value portion of token=value string
   private String getTokenValue(String tok, String tokname) {
        return tok.substring(tokname.length());
   }

//  method to locate a String entry in a Vector
   private int findElement(Vector v, String s) {
        for (int i=0; i < v.size() ; i++ ) {
           if ( s.equals( (String) v.elementAt(i) ) )
                return i;
        } /* endfor */
        return -1;
   }

//  method to return a data string
   private void outData(String data) {
        vOut.addElement(data);
   }

//  method to display a data string
   private void outMessage(String message) {
        System.out.println(message);
   }

//  method to read a file into a Vector
   private void readFile(String filename, Vector inCore) {
     String sTemp;
     boolean bEof = false;
     inCore.removeAllElements();

     try {
        File readFile = new File(filename);
        DataInputStream di = new DataInputStream( new FileInputStream(readFile) );

        while (!bEof) {
          sTemp = di.readLine().trim();
          if (sTemp != null) {
             inCore.addElement(sTemp);
             outMessage(sTemp + " loaded");
          }
          else
             bEof = true;
       }
       di.close();

    } /* endtry */
    catch (Exception e  ) {
//         outMessage(e.toString());
         return;
    } /* endcatch */
    return;

   }

//   method to write a Vector to a file
   private void writeData(String filename, Vector inCore) {
     String sTemp;
     boolean bEof = false;
     int    iCount = 0;

     if (inCore.size() == 0) {
        outData("nothing to write");
        outMessage("nothing to write");
        return;
     }

     try {
        File writeFile = new File(filename);
        PrintStream ps = new PrintStream( new FileOutputStream(writeFile) );

        for (int i=0; i<inCore.size(); i++) {
           ps.println( (String) inCore.elementAt(i) );
           iCount++;
        } /* endfor */
        ps.flush();
        ps.close();
        outData(iCount + " records written to " + filename);
    } /* endtry */
    catch (Exception e  ) {
//         outMessage(e.toString());
         return;
    } /* endcatch */
    return;

   }

}
